from celescope.__init__ import __CONDA__
from celescope.snp.__init__ import __STEPS__, __ASSAY__
from celescope.tools.Multi import Multi


class Multi_snp(Multi):
    def custome_args(self):
        self.STAR_args()
        self.parser.add_argument('--gene_list', help="gene_list", required=True)
        self.parser.add_argument('--probe_file', help="probe fasta file")
        self.parser.add_argument('--annovar_config', help='annovar soft config file')

    def read_custome_args(self):
        self.read_STAR_args()
        self.gene_list = self.args.gene_list
        self.probe_file = self.args.probe_file
        self.annovar_config = self.args.annovar_config

    def snpCalling(self, sample):
        step = 'snpCalling'
        bam = f'{self.outdir_dic[sample]["featureCounts"]}/{sample}_name_sorted.bam'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step} '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--bam {bam} '
            f'--match_dir {self.col4_dict[sample]} '
            f'--genomeDir {self.genomeDir} '
            f'--gene_list {self.gene_list} '
            f'--thread {self.thread} '
        )
        self.generate_other(cmd, step, sample, m=8, x=self.thread)

    def analysis_snp(self, sample):
        step = 'analysis_snp'
        vcf_anno = f'{self.outdir_dic[sample]["snpCalling"]}/{sample}_anno.vcf'
        index_file = f'{self.outdir_dic[sample]["snpCalling"]}/{sample}_cell_index.tsv'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step} '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--match_dir {self.col4_dict[sample]} '
            f'--vcf_anno {vcf_anno} '
            f'--index_file {index_file} '
            f'--annovar_config {self.annovar_config} '
        )
        self.generate_other(cmd, step, sample, m=8, x=self.thread)

    def run(self):
        self.multi_opts()
        self.custome_args()
        self.parse_args()
        self.read_custome_args()
        self.prepare()
        self.gtf_type = 'gene'
        self.run_steps()
        self.end()


def main():
    multi = Multi_snp(__ASSAY__, __STEPS__, __CONDA__)
    multi.col4_default = None
    multi.run()

if __name__ == '__main__':
    main()




from celescope.fusion.__init__ import __STEPS__, __ASSAY__


def run(args):
    steps = __STEPS__
    args.assay = __ASSAY__
    sample = args.sample

    outdir_dic = {}
    index = 0
    for step in steps:
        outdir = f"{sample}/{index:02d}.{step}"
        outdir_dic.update({step: outdir})
        index += 1

    step = "sample"
    args.outdir = f'{outdir_dic[step]}/'
    from celescope.tools.sample_info import sample_info
    sample_info(args)

    step = "barcode"
    args.outdir = f'{outdir_dic[step]}/'
    from celescope.tools.barcode import barcode
    barcode(args)

    step = "cutadapt"
    args.outdir = f'{outdir_dic[step]}/'
    args.fq = f'{outdir_dic["barcode"]}/{sample}_2.fq.gz'
    from celescope.tools.cutadapt import cutadapt
    cutadapt(args)

    step = "STAR_fusion"
    args.input_read = f'{outdir_dic["cutadapt"]}/{sample}_clean_2.fq.gz'
    args.outdir = f'{outdir_dic[step]}/'
    from celescope.fusion.STAR_fusion import STAR_fusion
    STAR_fusion(args)

    step = 'count_fusion'
    args.outdir = f'{outdir_dic[step]}/'
    args.bam = f'{outdir_dic["STAR_fusion"]}/{sample}_Aligned.sortedByCoord.out.bam'
    from celescope.fusion.count_fusion import count_fusion
    count_fusion(args)



import unittest
import os
import pandas as pd
from celescope.capture_rna.count_capture_rna import expression_matrix
from celescope.tools.utils import read_barcode_file, read_one_col, glob_genomeDir
from celescope.tools.report import reporter


class testHLA(unittest.TestCase):
    def setUp(self):
        os.chdir('/SGRNJ01/RD_dir/pipeline_test/zhouyiqi/0910_panel/')
        self.sample = 'S20071508_D_TS'
        count_detail_file = './/S20071508_D_TS/05.count_capture_rna/S20071508_D_TS_count_detail.txt'
        self.df = pd.read_table(count_detail_file, header=0)
        self.match_dir = '/SGRNJ02/RandD4/RD20051303_Panel/20200729/S20071508_D_ZL'
        self.sc_cell_barcodes, self.sc_cell_number = read_barcode_file(self.match_dir)
        self.outdir = f'{self.sample}/05.count_capture_rna/'
        self.genomeDir = '/SGRNJ/Public/Database/genome/homo_sapiens/ensembl_92'
        self.validated_barcodes, _ = read_one_col(f'{self.sample}/05.count_capture_rna/{self.sample}_matrix_10X/barcodes.tsv') 
        _refFlat, self.gtf = glob_genomeDir(self.genomeDir)
        self.assay = 'capture_rna'

    #@unittest.skip('pass')
    def test_expression_matrix(self):
        (
            CB_total_Genes,
            CB_reads_count,
            reads_mapped_to_transcriptome,
            match_cell_str,
            match_UMI_median
        ) = expression_matrix(
            self.df,
            self.validated_barcodes,
            self.outdir,
            self.sample,
            self.gtf,
            self.sc_cell_barcodes,
            self.sc_cell_number
            )
        print(match_cell_str)
        print(match_UMI_median)

    def test_report(self):
        t = reporter(assay=self.assay,
            name='count_capture_rna', sample=self.sample,
            stat_file=self.outdir + '/stat.txt',
            outdir=self.outdir + '/..')
        t.get_report()




if __name__ == '__main__':
    unittest.main()


from celescope.tools.report import reporter
from celescope.tools.utils import format_number, log, genDict
from celescope.tools.utils import read_fasta, process_read, gen_stat
from celescope.tools.barcode import parse_pattern
import gzip
import os
import pandas as pd
import logging
import numpy as np
import sys
import argparse
import re
import glob
from collections import defaultdict
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt


class Mapping_tag():

    def __init__(
        self,
        sample,
        outdir,
        assay,
        fq,
        fq_pattern,
        linker_fasta,
        barcode_fasta,
        ):

        # read args
        self.sample = sample
        self.outdir = outdir
        self.assay = assay
        self.fq = fq
        self.fq_pattern = fq_pattern
        self.linker_fasta = linker_fasta
        self.barcode_fasta = barcode_fasta

        # process
        self.barcode_dic, self.barcode_length = read_fasta(self.barcode_fasta, equal=True)
        if self.linker_fasta and self.linker_fasta != 'None':
            self.linker_dic, self.linker_length = read_fasta(self.linker_fasta, equal=True)
        else:
            self.linker_dic, self.linker_length = {}, 0
        self.pattern_dict = parse_pattern(self.fq_pattern)

        # check barcode length
        barcode1 = self.pattern_dict["C"][0]
        # end - start
        pattern_barcode_length = barcode1[1] - barcode1[0]
        if pattern_barcode_length != self.barcode_length:
            raise Exception(
                f'''barcode fasta length {self.barcode_length} 
                != pattern barcode length {pattern_barcode_length}'''
            )

        self.res_dic = genDict()
        self.res_sum_dic = genDict(dim=2)

        self.match_barcode = []
        self.read_count_file = f'{outdir}/{sample}_read_count.tsv'
        self.UMI_count_file = f'{outdir}/{sample}_UMI_count.tsv'
        self.stat_file = self.outdir + "/stat.txt"

        if not os.path.exists(outdir):
            os.system('mkdir -p %s' % outdir)

    def read_to_dic(self):

        self.res_dic, self.metrics = process_read(
            self.fq,
            self.pattern_dict,
            self.barcode_dic,
            self.linker_dic,
            self.barcode_length,
            self.linker_length,
        )

        # write dic to pandas df
        rows = []
        for barcode in self.res_dic:
            for tag_name in self.res_dic[barcode]:
                for umi in self.res_dic[barcode][tag_name]:
                    rows.append([barcode, tag_name, umi,
                                 self.res_dic[barcode][tag_name][umi]])
        df_read_count = pd.DataFrame(rows)
        df_read_count.rename(
            columns={
                0: "barcode",
                1: "tag_name",
                2: "UMI",
                3: "read_count"},
            inplace=True)
        df_read_count.to_csv(
            self.read_count_file, sep="\t", index=False)

        # stat
        stat = pd.DataFrame(
            {
                "item": [
                    "Reads Mapped",
                    "Reads Unmapped too Short",
                    "Reads Unmapped Invalid Linker",
                    "Reads Unmapped Invalid Barcode",
                ],
                "count": [
                    self.metrics["Reads Mapped"],
                    self.metrics["Reads Unmapped too Short"],
                    self.metrics["Reads Unmapped Invalid Linker"],
                    self.metrics["Reads Unmapped Invalid Barcode"],
                ],
            }, 
            columns=["item", "count"]
        )
        stat['total_count'] = self.metrics["Total Reads"]
        gen_stat(stat, self.stat_file)

    def tag_count(self):
        for barcode in self.res_dic:
            for tag_name in self.res_dic[barcode]:
                self.res_sum_dic[barcode][tag_name] = len(
                    self.res_dic[barcode][tag_name])

        df_umi_count = pd.DataFrame(self.res_sum_dic)
        df_umi_count = df_umi_count.T
        df_umi_count.fillna(0, inplace=True)
        df_umi_count = df_umi_count.astype(int)
        df_umi_count.to_csv(self.UMI_count_file, sep="\t")

    def report(self):
        t = reporter(
            name='mapping_tag', 
            assay=self.assay, 
            sample=self.sample,
            stat_file=self.stat_file, 
            outdir=self.outdir + '/..')
        t.get_report()

    @log
    def run(self):
        self.read_to_dic()
        self.tag_count()
        self.report()






from .Mapping_tag import Mapping_tag
import argparse

def get_opts_mapping_tag(parser, sub_program):
    parser.add_argument("--fq_pattern", help="read2 fastq pattern")
    parser.add_argument("--linker_fasta", help="linker fasta")
    parser.add_argument("--barcode_fasta", help="barcode fasta")
    if sub_program:
        parser.add_argument("--fq", help="clean read2", required=True)
        parser.add_argument('--outdir', help='output dir', required=True)
        parser.add_argument('--sample', help='sample name', required=True)
        parser.add_argument('--assay', help='assay', required=True)


def mapping_tag(args):
    sample = args.sample
    outdir = args.outdir
    assay = args.assay
    fq = args.fq
    fq_pattern = args.fq_pattern
    linker_fasta = args.linker_fasta
    barcode_fasta = args.barcode_fasta

    mapping_tag_object = Mapping_tag(
        sample,
        outdir,
        assay,
        fq,
        fq_pattern,
        linker_fasta,
        barcode_fasta,
    )
    mapping_tag_object.run()



#!/bin/env python
# coding=utf8
import os
import re
import io
import logging
import gzip
import json
import subprocess
from collections import defaultdict, namedtuple
import sys


def run(args):
    steps = [
        'sample',
        'barcode',
        'cutadapt',
        "STAR_virus",
        "count_capture_virus"]
    sample = args.sample
    args.assay = "capture_virus"

    outdir_dic = {}
    index = 0
    for step in steps:
        outdir = f"{sample}/{index:02d}.{step}"
        outdir_dic.update({step: outdir})
        index += 1

    step = "sample"
    args.outdir = f'{outdir_dic[step]}/'
    from celescope.tools.sample_info import sample_info
    sample_info(args)

    step = "barcode"
    args.outdir = f'{outdir_dic[step]}/'
    from celescope.tools.barcode import barcode
    barcode(args)

    step = "cutadapt"
    args.outdir = f'{outdir_dic[step]}/'
    args.fq = f'{outdir_dic["barcode"]}/{sample}_2.fq.gz'
    from celescope.tools.cutadapt import cutadapt
    cutadapt(args)

    step = "STAR_virus"
    args.input_read = f'{outdir_dic["cutadapt"]}/{sample}_clean_2.fq.gz'
    args.outdir = f'{outdir_dic[step]}/'
    from celescope.rna_virus.STAR_virus import STAR_virus
    STAR_virus(args)

    step = 'count_capture_virus'
    args.virus_bam = f'{outdir_dic["STAR_virus"]}/{sample}_virus_Aligned.sortedByCoord.out.bam'
    args.outdir = f'{outdir_dic[step]}/'
    from celescope.capture_virus.count_capture_virus import count_capture_virus
    count_capture_virus(args)



#!/bin/env python
# coding=utf8
import os
import re
import io
import gzip
import subprocess
import sys
import glob
import pandas as pd
import pysam
from collections import defaultdict, Counter
from itertools import combinations, permutations, islice
from xopen import xopen
from celescope.tools.utils import format_number, log, seq_ranges, read_fasta, genDict, read_one_col
from celescope.tools.report import reporter
from celescope.tools.__init__ import __PATTERN_DICT__

class Barcode():
    def __init__(self, fq1, fq2, sample, outdir): 
        # TODO
        self.read = 0
        self.nRead = 10000
        self.T4 = 0
        self.L57C = 0
        self.fq1_file, self.fq2_file = self.fq1, self.fq2

    @log
    def merge_fastq(self):
        '''
        merge fastq if len(fq1) > 1
        '''
        fq1_list = self.fq1.split(",")
        fq2_list = self.fq2.split(",")
        if len(fq1_list) == 0:
            raise Exception('empty fastq file path!')
        elif len(fq1_list) == 1:
            self.fq1_file, self.fq2_file = self.fq1, self.fq2
        else:
            fastq_dir = f'{self.outdir}/../merge_fastq'
            if not os.path.exists(fastq_dir):
                os.system('mkdir -p %s' % fastq_dir)
            fq1_file = f"{fastq_dir}/{self.sample}_1.fq.gz"
            fq2_file = f"{fastq_dir}/{self.sample}_2.fq.gz"
            fq1_files = " ".join(fq1_list)
            fq2_files = " ".join(fq2_list)
            fq1_cmd = f"cat {fq1_files} > {fq1_file}"
            fq2_cmd = f"cat {fq2_files} > {fq2_file}"
            Barcode.merge_fastq.logger.info(fq1_cmd)
            os.system(fq1_cmd)
            Barcode.merge_fastq.logger.info(fq2_cmd)
            os.system(fq2_cmd)
            self.fq1_file = fq1_file
            self.fq2_file = fq2_file
        return self.fq1_file, self.fq2_file

    '''
    @log
    def get_chemistry(self):
        '''
        'scopeV2.0.1': 'C8L16C8L16C8L1U8T18'
        'scopeV2.1.1': 'C8L16C8L16C8L1U12T18'
        'scopeV2.2.1': 'C8L16C8L16C8L1U12T18' with 4 types of linkers
        '''
        # init
        linker_4_file, _whitelist = Barcode.get_scope_bc('scopeV2.2.1')
        linker_4_list, _num = read_one_col(linker_4_file)
        linker_4_dict = defaultdict(int)
        pattern_dict = parse_pattern('C8L16C8L16C8L1U12T18')

        with pysam.FastxFile(self.fq1) as fh:
            for i in range(self.nRead):
                entry = fh.__next__()
                seq = entry.sequence
                L57C = seq[56]
                if L57C == 'C':
                    self.L57C += 1
                T4 = seq[65:69]
                if T4 == 'TTTT':
                    self.T4 += 1
                linker = seq_ranges(seq, pattern_dict=pattern_dict)
                if linker in linker_4_list:
                    linker_4_dict[linker] += 1

        self.percent_T4 =  self.T4 / self.nRead 
        self.percent_L57C = self.L57C / self.nRead
        Chemistry.get_chemistry.logger.info(f'percent T4: {self.percent_T4}')
        Chemistry.get_chemistry.logger.info(f'percent L57C: {self.percent_L57C}')
        if self.percent_T4 > 0.5:
            self.chemistry = 'scopeV2.0.1'
        else:
            # V2.1.1 or V2.2.1 or failed
            self.valid_linker_type = 0
            for linker in linker_4_dict:
                linker_4_dict[linker] = linker_4_dict[linker] / self.nRead
                if linker_4_dict[linker] > 0.05:
                    self.valid_linker_type += 1
            Chemistry.get_chemistry.logger.info(self.linker_4_dict)
            if self.valid_linker_type == 0:
                raise Exception('auto chemistry detection failed!')
            elif self.valid_linker_type == 1:
                self.chemistry == 'scopeV2.1.1'
            elif self.valid_linker_type < 4:
                self.chemistry == 'scopeV2.1.1'
                Chemistry.get_chemistry.logger.warning(
                    f'chemistry scopeV2.2.1 only has {self.valid_linker_type} linker types!')
            else:
                self.chemistry == 'scopeV2.2.1'
        Chemistry.get_chemistry.logger.info(f'chemistry: {self.chemistry}')
        return self.chemistry
    '''

    @staticmethod
    def get_scope_bc(bctype):
        import celescope
        root_path = os.path.dirname(celescope.__file__)
        linker_f = glob.glob(f'{root_path}/data/chemistry/{bctype}/linker*')[0]
        whitelist_f = f'{root_path}/data/chemistry/{bctype}/bclist'
        return linker_f, whitelist_f


#!/bin/env python
# coding=utf8
import os
import re
import io
import gzip
import subprocess
import sys
import glob
import pandas as pd
from collections import defaultdict, Counter
from itertools import combinations, permutations, islice
from xopen import xopen
from celescope.tools.utils import format_number, log, seq_ranges, read_fasta, genDict
from celescope.tools.report import reporter
from celescope.tools.__init__ import __PATTERN_DICT__
from .Chemistry import Chemistry

barcode_corrected_num = 0

# 定义输出格式
stat_info = '''
    Raw Reads: %s
    Valid Reads: %s(%s)
    Q30 of Barcodes: %.2f%%
    Q30 of UMIs: %.2f%%
'''


def ord2chr(q, offset=33):
    return chr(int(q) + offset)


# 生成错配字典
def generate_mis_seq(seq, n=1, bases='ACGTN'):
    # 以随机bases中的碱基替换seq中的n个位置，产生的错配字典
    # 返回字典，错配序列为key，
    # (正确序列，错配碱基数目，错配碱基位置，原始碱基，新碱基)组成的元组
    # 作为字典的值

    length = len(seq)
    assert length >= n, "err number should not be larger than sequence length!"
    res = {}
    seq_arr = list(seq)
    pos_group = list(combinations(range(0, length), n))
    bases_group = list(permutations(bases, n))

    for g in pos_group:
        for b in bases_group:
            seq_tmp = seq_arr[:]
            mis_num = n
            raw_tmp = []
            for i in range(n):
                raw_base = seq_tmp[g[i]]
                new_base = b[i]

                if raw_base == new_base:
                    mis_num -= 1

                raw_tmp.append(raw_base)
                seq_tmp[g[i]] = new_base

            if mis_num != 0:
                res[''.join(seq_tmp)] = (seq, mis_num, ','.join(
                    [str(i) for i in g]), ','.join(raw_tmp), ','.join(b))
    return(res)


@log
def generate_seq_dict(seqlist, n=1):
    seq_dict = {}
    with open(seqlist, 'r') as fh:
        for seq in fh:
            seq = seq.strip()
            if seq == '':
                continue
            seq_dict[seq] = (seq, 0, -1, 'X', 'X')
            for k, v in generate_mis_seq(seq, n).items():
                # duplicate key
                if k in seq_dict:
                    generate_seq_dict.logger.warning('barcode %s, %s\n%s, %s' %
                                                     (v, k, seq_dict[k], k))
                else:
                    seq_dict[k] = v
    return seq_dict


@log
def parse_pattern(pattern):
    # 解析接头结构，返回接头结构字典
    # key: 字母表示的接头, value: 碱基区间列表
    # eg.: C8L10C8L10C8U8T30
    # defaultdict(<type 'list'>:
    # {'C': [[0, 8], [18, 26], [36, 44]], 'U': [[44, 52]], 'L': [[8, 18], [26, 36]], 'T': [[52, 82]]})
    pattern_dict = defaultdict(list)
    p = re.compile(r'([CLUNT])(\d+)')
    tmp = p.findall(pattern)
    if not tmp:
        parse_pattern.logger.error(f'Invalid pattern: {pattern}')
        sys.exit()
    start = 0
    for item in tmp:
        end = start + int(item[1])
        pattern_dict[item[0]].append([start, end])
        start = end
    return pattern_dict


def get_scope_bc(bctype):
    import celescope
    root_path = os.path.dirname(celescope.__file__)
    linker_f = glob.glob(f'{root_path}/data/chemistry/{bctype}/linker*')[0]
    whitelist_f = f'{root_path}/data/chemistry/{bctype}/bclist'
    return linker_f, whitelist_f


def read_fastq(f):
    """
    Return tuples: (name, sequence, qualities).
    qualities is a string and it contains the unmodified, encoded qualities.
    """
    i = 3
    for i, line in enumerate(f):
        if i % 4 == 0:
            assert line.startswith('@'), ("Line {0} in FASTQ file is expected to start with '@', "
                                          "but found {1!r}".format(i + 1, line[:10]))
            name = line.strip()[1:]
        elif i % 4 == 1:
            sequence = line.strip()
        elif i % 4 == 2:
            line = line.strip()
            assert line.startswith('+'), ("Line {0} in FASTQ file is expected to start with '+', "
                                          "but found {1!r}".format(i + 1, line[:10]))
        elif i % 4 == 3:
            qualities = line.rstrip('\n\r')
            yield name, sequence, qualities
    if i % 4 != 3:
        raise Exception("FASTQ file ended prematurely")


def low_qual(quals, minQ='/', num=2):
    # print(ord('/')-33)           14
    return True if len([q for q in quals if q < minQ]) > num else False


def no_polyT(seq, strictT=0, minT=10):
    # strictT requires the first nth position to be T
    if seq[:strictT] != 'T' * strictT or seq.count('T') < minT:
        return True
    else:
        return False


def no_barcode(seq_arr, mis_dict, err_tolerance=1):
    global barcode_corrected_num
    tmp = [mis_dict[seq][0:2] if seq in mis_dict else (
        'X', 100) for seq in seq_arr]
    err = sum([t[1] for t in tmp])
    if err > err_tolerance:
        return True
    else:
        if err > 0:
            barcode_corrected_num += 1
            return ''.join([t[0] for t in tmp])
        else:
            return "correct"


def no_linker(seq, linker_dict):
    return False if seq in linker_dict else True


def check_seq(seq_file, pattern_dict, seq_abbr):
    length = 0
    for item in pattern_dict[seq_abbr]:
        start = item[0]
        end = item[1]
        length += end - start
    with open(seq_file, 'r') as fh:
        for seq in fh:
            seq = seq.strip()
            if seq == '':
                continue
            if len(seq) != length:
                raise Exception(
                    f'length of L in pattern ({length}) do not equal to length in {seq_file} ({len(seq)}) !')


@log
def merge_fastq(fq1, fq2, sample, outdir):
    '''
    merge fastq if len(fq1) > 1
    '''
    fq1_list = fq1.split(",")
    fq2_list = fq2.split(",")
    if len(fq1_list) == 0:
        raise Exception('empty fastq file path!')
    elif len(fq1_list) == 1:
        fq1_file, fq2_file = fq1, fq2
    else:
        fastq_dir = f'{outdir}/../merge_fastq'
        if not os.path.exists(fastq_dir):
            os.system('mkdir -p %s' % fastq_dir)
        fq1_file = f"{fastq_dir}/{sample}_1.fq.gz"
        fq2_file = f"{fastq_dir}/{sample}_2.fq.gz"
        fq1_files = " ".join(fq1_list)
        fq2_files = " ".join(fq2_list)
        fq1_cmd = f"cat {fq1_files} > {fq1_file}"
        fq2_cmd = f"cat {fq2_files} > {fq2_file}"
        merge_fastq.logger.info(fq1_cmd)
        os.system(fq1_cmd)
        merge_fastq.logger.info(fq2_cmd)
        os.system(fq2_cmd)
    return fq1_file, fq2_file
 

@log
def barcode(args):
    # init
    outdir = args.outdir
    sample = args.sample
    fq1 = args.fq1
    fq2 = args.fq2

    # check dir
    if not os.path.exists(args.outdir):
        os.system('mkdir -p %s' % args.outdir)

    # merge fastq
    fq1_file, fq2_file = merge_fastq(fq1, fq2, sample, outdir)

    # get chemistry
    if args.chemistry == 'auto':
        ch = Chemistry(fq1_file)
        chemistry = ch.get_chemistry()
    else:
        chemistry = args.chemistry

    # get linker and whitelist
    bc_pattern = __PATTERN_DICT__[chemistry]
    if (bc_pattern):
        (linker, whitelist) = get_scope_bc(chemistry)
    else:
        bc_pattern = args.pattern
        linker = args.linker
        whitelist = args.whitelist
    if (not linker) or (not whitelist) or (not bc_pattern):
        raise Exception("invalid chemistry or [pattern,linker,whitelist]")
        
    # parse pattern to dict, C8L10C8L10C8U8
    # defaultdict(<type 'list'>, {'C': [[0, 8], [18, 26], [36, 44]], 'U':
    # [[44, 52]], 'L': [[8, 18], [26, 36]]})
    pattern_dict = parse_pattern(bc_pattern)

    # check linker
    check_seq(linker, pattern_dict, "L")

    bool_T = True if 'T' in pattern_dict else False
    bool_L = True if 'L' in pattern_dict else False

    C_len = sum([item[1] - item[0] for item in pattern_dict['C']])

    barcode_qual_Counter = Counter()
    umi_qual_Counter = Counter()
    C_U_base_Counter = Counter()
    args.lowQual = ord2chr(args.lowQual)

    # generate list with mismatch 1, substitute one base in raw sequence with
    # A,T,C,G
    barcode_dict = generate_seq_dict(whitelist, n=1)
    linker_dict = generate_seq_dict(linker, n=2)

    fh1 = xopen(fq1_file)
    fh2 = xopen(fq2_file)
    out_fq2 = args.outdir + '/' + args.sample + '_2.fq.gz'
    fh3 = xopen(out_fq2, 'w')

    (total_num, clean_num, no_polyT_num, lowQual_num,
     no_linker_num, no_barcode_num) = (0, 0, 0, 0, 0, 0)
    Barcode_dict = defaultdict(int)

    if args.nopolyT:
        fh1_without_polyT = xopen(outdir + '/noPolyT_1.fq', 'w')
        fh2_without_polyT = xopen(outdir + '/noPolyT_2.fq', 'w')

    if args.noLinker:
        fh1_without_linker = xopen(outdir + '/noLinker_1.fq', 'w')
        fh2_without_linker = xopen(outdir + '/noLinker_2.fq', 'w')

    bool_probe = False
    if args.probe_file and args.probe_file != 'None':
        bool_probe = True
        count_dic = genDict(dim=3)
        valid_count_dic = genDict(dim=2)
        probe_dic = read_fasta(args.probe_file)
        reads_without_probe = 0

    g1 = read_fastq(fh1)
    g2 = read_fastq(fh2)

    while True:
        try:
            (header1, seq1, qual1) = next(g1)
            (header2, seq2, qual2) = next(g2)
        except BaseException:
            break
        if total_num > 0 and total_num % 1000000 == 0:
            barcode.logger.info(
                f'processed reads: {format_number(total_num)}.'
                f'valid reads: {format_number(clean_num)}.'
            )

        total_num += 1

        # polyT filter
        if bool_T:
            polyT = seq_ranges(seq1, pattern_dict['T'])
            if no_polyT(polyT):
                no_polyT_num += 1
                if args.nopolyT:
                    fh1_without_polyT.write(
                        '@%s\n%s\n+\n%s\n' % (header1, seq1, qual1))
                    fh2_without_polyT.write(
                        '@%s\n%s\n+\n%s\n' % (header2, seq2, qual2))
                continue

        # lowQual filter
        C_U_quals_ascii = seq_ranges(
            qual1, pattern_dict['C'] + pattern_dict['U'])
        # C_U_quals_ord = [ord(q) - 33 for q in C_U_quals_ascii]
        if low_qual(C_U_quals_ascii, args.lowQual, args.lowNum):
            lowQual_num += 1
            continue

        # linker filter
        barcode_arr = [seq_ranges(seq1, [i]) for i in pattern_dict['C']]
        raw_cb = ''.join(barcode_arr)
        if bool_L:
            linker = seq_ranges(seq1, pattern_dict['L'])
            if (no_linker(linker, linker_dict)):
                no_linker_num += 1

                if args.noLinker:
                    fh1_without_linker.write(
                        '@%s\n%s\n+\n%s\n' % (header1, seq1, qual1))
                    fh2_without_linker.write(
                        '@%s\n%s\n+\n%s\n' % (header2, seq2, qual2))
                continue

        # barcode filter
        # barcode_arr = [seq_ranges(seq1, [i]) for i in pattern_dict['C']]
        # raw_cb = ''.join(barcode_arr)
        res = no_barcode(barcode_arr, barcode_dict)
        if res is True:
            no_barcode_num += 1
            continue
        elif res == "correct":
            cb = raw_cb
        else:
            cb = res

        umi = seq_ranges(seq1, pattern_dict['U'])
        Barcode_dict[cb] += 1
        clean_num += 1
        read_name_probe = 'None'

        if bool_probe:
            # valid count
            valid_count_dic[cb][umi] += 1

            # output probe UMi and read count
            find_probe = False
            for probe_name in probe_dic:
                probe_seq = probe_dic[probe_name]
                probe_seq = probe_seq.upper()
                if seq1.find(probe_seq) != -1:
                    count_dic[probe_name][cb][umi] += 1
                    read_name_probe = probe_name
                    find_probe = True
                    break

            if not find_probe:
                reads_without_probe += 1

        barcode_qual_Counter.update(C_U_quals_ascii[:C_len])
        umi_qual_Counter.update(C_U_quals_ascii[C_len:])
        C_U_base_Counter.update(raw_cb + umi)

        # new readID: @barcode_umi_old readID
        fh3.write(f'@{cb}_{umi}_{read_name_probe}_{total_num}\n{seq2}\n+\n{qual2}\n')

    fh3.close()

    # logging
    if total_num % 1000000 != 0:
        barcode.logger.info(
            f'processed reads: {format_number(total_num)}. '
            f'valid reads: {format_number(clean_num)}. '
        )

    barcode.logger.info(f'no_linker: {no_linker_num}')
    barcode.logger.info(f'no_barcode: {no_barcode_num}')
    barcode.logger.info(f'no_polyT: {no_polyT_num}')

    if clean_num == 0:
        raise Exception(
            'no valid reads found! please check the --chemistry parameter.')

    if bool_probe:
        # total probe summary
        total_umi = 0
        total_valid_read = 0
        for cb in valid_count_dic:
            total_umi += len(valid_count_dic[cb])
            total_valid_read += sum(valid_count_dic[cb].values())
        barcode.logger.info("total umi:"+str(total_umi))
        barcode.logger.info("total valid read:"+str(total_valid_read))
        barcode.logger.info("reads without probe:"+str(reads_without_probe))

        # probe summary
        count_list = []
        for probe_name in probe_dic:
            UMI_count = 0
            read_count = 0
            if probe_name in count_dic:
                for cb in count_dic[probe_name]:
                    UMI_count += len(count_dic[probe_name][cb])
                    read_count += sum(count_dic[probe_name][cb].values())
            count_list.append(
                {"probe_name": probe_name, "UMI_count": UMI_count, "read_count": read_count})

        df_count = pd.DataFrame(count_list, columns=[
                                "probe_name", "read_count", "UMI_count"])

        def format_percent(x):
            x = str(round(x*100, 2))+"%"
            return x
        df_count["read_fraction"] = (
            df_count["read_count"]/total_valid_read).apply(format_percent)
        df_count["UMI_fraction"] = (
            df_count["UMI_count"]/total_umi).apply(format_percent)
        df_count.sort_values(by="UMI_count", inplace=True, ascending=False)
        df_count_file = args.outdir + '/' + args.sample + '_probe_count.tsv'
        df_count.to_csv(df_count_file, sep="\t", index=False)

    # stat
    BarcodesQ30 = sum([barcode_qual_Counter[k] for k in barcode_qual_Counter if k >= ord2chr(
        30)]) / float(sum(barcode_qual_Counter.values())) * 100
    UMIsQ30 = sum([umi_qual_Counter[k] for k in umi_qual_Counter if k >= ord2chr(
        30)]) / float(sum(umi_qual_Counter.values())) * 100

    global stat_info
    def cal_percent(x): return "{:.2%}".format((x + 0.0) / total_num)
    with open(args.outdir + '/stat.txt', 'w') as fh:
        """
        Raw Reads: %s
        Valid Reads: %s(%s)
        Q30 of Barcodes: %.2f%%
        Q30 of UMIs: %.2f%%
        """
        stat_info = stat_info % (format_number(total_num), format_number(clean_num),
                                 cal_percent(clean_num), BarcodesQ30,
                                 UMIsQ30)
        stat_info = re.sub(r'^\s+', r'', stat_info, flags=re.M)
        fh.write(stat_info)

    barcode.logger.info('fastqc ...!')
    cmd = ['fastqc', '-t', str(args.thread), '-o', args.outdir, out_fq2]
    barcode.logger.info('%s' % (' '.join(cmd)))
    subprocess.check_call(cmd)
    barcode.logger.info('fastqc done!')

    t = reporter(name='barcode', assay=args.assay, sample=args.sample,
                 stat_file=args.outdir + '/stat.txt', outdir=args.outdir + '/..')
    t.get_report()


def get_opts_barcode(parser, sub_program):
    if sub_program:
        parser.add_argument('--outdir', help='output dir', required=True)
        parser.add_argument('--assay', help='assay', required=True)
    parser.add_argument('--sample', help='sample name', required=True)
    parser.add_argument('--fq1', help='read1 fq file', required=True)
    parser.add_argument('--fq2', help='read2 fq file', required=True)
    parser.add_argument(
        '--chemistry', choices=__PATTERN_DICT__.keys(), help='chemistry version')
    parser.add_argument('--pattern', help='')
    parser.add_argument('--whitelist', help='')
    parser.add_argument('--linker', help='')
    parser.add_argument('--lowQual', type=int,
                        help='max phred of base as lowQual, default=0', default=0)
    parser.add_argument(
        '--lowNum', type=int, help='max number with lowQual allowed, default=2', default=2)
    parser.add_argument('--nopolyT', action='store_true',
                        help='output nopolyT fq')
    parser.add_argument('--noLinker', action='store_true',
                        help='output noLinker fq')
    parser.add_argument('--thread', help='number of threads', default=1)
    parser.add_argument('--probe_file', help="probe fasta file")
    return parser



#!/bin/env python
# coding=utf8
import os
import re
import io
import logging
import gzip
import json
import subprocess
from collections import defaultdict, namedtuple
import sys


def run(args):
    steps = ['sample', 'barcode', 'cutadapt', 'STAR', "STAR_virus", "featureCounts", "count", "count_virus",
             'analysis']
    sample = args.sample
    args.assay = "rna_virus"

    outdir_dic = {}
    index = 0
    for step in steps:
        outdir = f"{sample}/{index:02d}.{step}"
        outdir_dic.update({step: outdir})
        index += 1

    step = "sample"
    args.outdir = f'{outdir_dic[step]}/'
    from celescope.tools.sample_info import sample_info
    sample_info(args)

    step = "barcode"
    args.outdir = f'{outdir_dic[step]}/'
    from celescope.tools.barcode import barcode
    barcode(args)

    step = "cutadapt"
    args.outdir = f'{outdir_dic[step]}/'
    args.fq = f'{outdir_dic["barcode"]}/{sample}_2.fq.gz'
    from celescope.tools.cutadapt import cutadapt
    cutadapt(args)

    step = "STAR"
    args.fq = f'{outdir_dic["cutadapt"]}/{sample}_clean_2.fq.gz'
    args.outdir = f'{outdir_dic[step]}/'
    args.out_unmapped = True
    from celescope.tools.STAR import STAR
    STAR(args)

    step = "STAR_virus"
    args.input_read = f'{outdir_dic["STAR"]}/{sample}_Unmapped.out.mate1'
    args.outdir = f'{outdir_dic[step]}/'
    from celescope.rna_virus.STAR_virus import STAR_virus
    STAR_virus(args)

    step = 'featureCounts'
    args.input = f'{outdir_dic["STAR"]}/{sample}_Aligned.sortedByCoord.out.bam'
    args.outdir = f'{outdir_dic[step]}/'
    from celescope.tools.featureCounts import featureCounts
    featureCounts(args)

    step = 'count'
    args.bam = f'{outdir_dic["featureCounts"]}/{sample}_name_sorted.bam'
    args.outdir = f'{outdir_dic[step]}/'
    from celescope.tools.count import count
    count(args)

    step = 'count_virus'
    args.virus_bam = f'{outdir_dic["STAR_virus"]}/{sample}_virus_Aligned.sortedByCoord.out.bam'
    args.barcode_file = f'{outdir_dic["count"]}/matrix_10X/{sample}_cellbarcode.tsv'
    args.outdir = f'{outdir_dic[step]}/'
    from celescope.virus.count_virus import count_virus
    count_virus(args)

    step = 'analysis'
    args.matrix_file = f'{outdir_dic["count"]}/{sample}_matrix.xls'
    args.virus_file = f'{outdir_dic["virus_count"]}/{sample}_virus_UMI_count.tsv'
    args.outdir = f'{outdir_dic[step]}/'
    from celescope.tools.analysis import analysis
    analysis(args)



import unittest
import os
import pandas as pd
from celescope.fusion.count_fusion import read_pos


class testHLA(unittest.TestCase):
    def setUp(self):
        os.chdir('/SGRNJ01/RD_dir/pipeline_test/zhouyiqi/CeleScope-Fusion/sjm_0818')
        self.fusion_pos = '/SGRNJ01/RD_dir/pipeline_test/zhouyiqi/CeleScope-Fusion/pipe_0722/fusion_pos.txt'

    def test_read_pos(self):
        print(read_pos(self.fusion_pos))


if __name__ == '__main__':
    unittest.main()



import os
from collections import defaultdict
import celescope
import pysam
import numpy as np
import pandas as pd
from celescope.tools.utils import format_number, log, read_barcode_file
from celescope.tools.utils import format_stat
from celescope.tools.utils import read_one_col, gene_convert, glob_genomeDir
from celescope.tools.report import reporter


class SplitBAM():

    def __init__(self, sample, outdir, bam, barcodes, gtf, gene_list_file=None):
        self.sample = sample
        self.outdir = outdir
        self.bam = bam
        self.barcodes = barcodes
        self.gtf = gtf
        self.gene_list_file = self.gene_list_file

    def umi_correction():
        pass

    def get_cell_index():
        pass

    @classmethod
    def get_gene_id_name_dict():
        pass   

        

    def split_bam(bam, barcodes, outdir, sample, gene_id_name_dic, min_query_length):
        '''
        input:
            bam: bam from feauturCounts
            barcodes: cell barcodes, set
            gene_id_name_dic: id name dic
            min_query_length: minimum query length

        ouput:
            bam_dict: assign reads to cell barcodes and UMI
            count_dict: UMI counts per cell
            index: assign index(1-based) to cells
        '''

        # init
        count_dict = defaultdict(dict)
        bam_dict = defaultdict(list)
        index_dict = defaultdict(dict)
        cells_dir = f'{outdir}/cells/'

        # read bam and split
        split_bam.logger.info('reading bam...')
        samfile = pysam.AlignmentFile(bam, "rb")
        header = samfile.header
        barcodes = list(barcodes)
        barcodes.sort()
        for read in samfile:
            attr = read.query_name.split('_')
            barcode = attr[0]
            umi = attr[1]
            if not read.has_tag('XT'):
                continue
            gene = read.get_tag('XT')
            query_length = read.infer_query_length()
            if (barcode in barcodes) and (gene in gene_id_name_dic) and (query_length >= min_query_length):
                gene_name = gene_id_name_dic[gene]
                read.set_tag(tag='GN', value=gene_name, value_type='Z')
                index = barcodes.index(barcode) + 1
                read.set_tag(tag='CL', value=f'CELL{index}', value_type='Z')

                # assign read to barcode
                bam_dict[barcode].append(read)

                # count
                if gene_name not in count_dict[barcode]:
                    count_dict[barcode][gene_name] = {}
                if umi in count_dict[barcode][gene_name]:
                    count_dict[barcode][gene_name][umi] += 1
                else:
                    count_dict[barcode][gene_name][umi] = 1

        split_bam.logger.info('writing cell bam...')
        # write new bam
        index = 0
        for barcode in barcodes:
            # init
            index += 1
            index_dict[index]['barcode'] = barcode
            index_dict[index]['valid'] = False

            # out bam
            if barcode in bam_dict:
                cell_dir = f'{cells_dir}/cell{index}'
                cell_bam_file = f'{cell_dir}/cell{index}.bam'
                if not os.path.exists(cell_dir):
                    os.makedirs(cell_dir)
                index_dict[index]['valid'] = True
                cell_bam = pysam.AlignmentFile(
                    f'{cell_bam_file}', "wb", header=header)
                for read in bam_dict[barcode]:
                    cell_bam.write(read)
                cell_bam.close()

        # out df_index
        df_index = pd.DataFrame(index_dict).T
        df_index.index.name = 'cell_index'
        index_file = f'{outdir}/{sample}_cell_index.tsv'
        df_index.to_csv(index_file, sep='\t')

        # out count_dict
        df_count = pd.DataFrame(columns=['barcode', 'gene', 'UMI', 'read_count'])
        for barcode in count_dict:
            for gene in count_dict[barcode]:
                for umi in count_dict[barcode][gene]:
                    read_count = count_dict[barcode][gene][umi]
                    df_count = df_count.append({
                        'barcode': barcode,
                        'gene': gene,
                        'UMI': umi,
                        'read_count': read_count,
                    }, ignore_index=True)
        count_file = f'{outdir}/{sample}_count.tsv'
        df_count.to_csv(count_file, sep='\t', index=False)

        return index_file, count_file


#!/bin/env python
# coding=utf8

import os
import sys
import re
import subprocess
import logging
from itertools import islice
import pandas as pd

from celescope.tools.utils import format_number, log
from celescope.tools.report import reporter


def format_stat(cutadapt_log, samplename):
    fh = open(cutadapt_log, 'r')
    stat_file = os.path.dirname(cutadapt_log) + '/stat.txt'
    # Total reads processed:...Total written (filtered):
    content = islice(fh, 9, 16)
    p_list = []
    for line in content:
        if line.strip() == '':
            continue
        line = re.sub(r'\s{2,}', r'', line)
        line = re.sub(r' bp', r'', line)
        line = re.sub(r'(?<=\d)\s+\(', r'(', line)
        line = line.strip()
        attr = line.split(":")
        p_list.append({"item": attr[0], "value": attr[1]})
    p_df = pd.DataFrame(p_list)
    p_df.iloc[0, 0] = 'Reads with Adapters'
    p_df.iloc[1, 0] = 'Reads too Short'
    p_df.iloc[2, 0] = 'Reads Written'
    p_df.iloc[3, 0] = 'Base Pairs Processed'
    p_df.iloc[4, 0] = 'Base Pairs Quality-Trimmed'
    p_df.iloc[5, 0] = 'Base Pairs Written'
    p_df.to_csv(stat_file, sep=':', index=False, header=None)

    fh.close()


@log
def cutadapt(args):
    # check dir
    if not os.path.exists(args.outdir):
        os.system('mkdir -p %s' % (args.outdir))

    # run cutadapt
    adapt = []
    for a in args.adapt:
        adapt.append('-a')
        adapt.append(a)

    out_fq2 = args.outdir + '/' + args.sample + '_clean_2.fq.gz'
    cmd = ['cutadapt'] + adapt + ['-n',
                                  str(len(args.adapt)),
                                  '-j',
                                  str(args.thread),
                                  '-m',
                                  str(args.minimum_length),
                                  '--nextseq-trim=' + str(args.nextseq_trim),
                                  '--overlap',
                                  str(args.overlap),
                                  '-o',
                                  out_fq2,
                                  args.fq]
    cutadapt.logger.info('%s' % (' '.join(cmd)))
    res = subprocess.run(cmd, stderr=subprocess.STDOUT, stdout=subprocess.PIPE)
    with open(args.outdir + '/cutadapt.log', 'wb') as fh:
        fh.write(res.stdout)

    format_stat(args.outdir + '/cutadapt.log', args.sample)
    t = reporter(
        name='cutadapt',
        assay=args.assay,
        sample=args.sample,
        stat_file=args.outdir +
        '/stat.txt',
        outdir=args.outdir +
        '/..')
    t.get_report()


def get_opts_cutadapt(parser, sub_program):
    if sub_program:
        parser.add_argument('--fq', help='fq file', required=True)
        parser.add_argument('--outdir', help='output dir', required=True)
        parser.add_argument('--sample', help='sample name', required=True)
        parser.add_argument('--assay', help='assay', required=True)
    parser.add_argument(
        '--adapt',
        action='append',
        default=[
            'polyT=A{18}',
            'p5=AGATCGGAAGAGCACACGTCTGAACTCCAGTCAC',
        ])
    parser.add_argument(
        '--minimum_length',
        dest='minimum_length',
        help='minimum_length, default=20',
        default=20)
    parser.add_argument(
        '--nextseq-trim',
        dest='nextseq_trim',
        help='nextseq_trim, default=20',
        default=20)
    parser.add_argument(
        '--overlap',
        help='minimum overlap length',
        default=10)
    parser.add_argument('--thread', default=2)



from celescope.vdj.__init__ import CHAINS
from celescope.tools.report import reporter
from celescope.tools.utils import format_number, gen_stat, log
import os
import logging
import gzip
import numpy as np
import pandas as pd
import matplotlib as mpl
import re
import json
import argparse
mpl.use('Agg')
from matplotlib import pyplot as plt


def summary(fq, alignments, type, outdir, sample, assay, debug):
    chains = CHAINS[type]

    '''
    # out files
    UMI_unfiltered_file = f'{outdir}/{sample}_UMI_unfiltered.tsv'
    UMI_filtered1_file = f'{outdir}/{sample}_UMI_filtered1.tsv'
    UMI_filtered2_file = f'{outdir}/{sample}_UMI_filtered2.tsv'
    '''

    UMI_count_unfiltered_file = f'{outdir}/{sample}_UMI_count_unfiltered.tsv'
    UMI_count_filtered1_file = f'{outdir}/{sample}_UMI_count_filtered1.tsv'

    # read fq
    read2 = gzip.open(fq, "rt")
    index = 0
    read_row_list = []
    while True:
        line1 = read2.readline()
        line2 = read2.readline()
        line3 = read2.readline()
        line4 = read2.readline()
        if not line4:
            break
        attr = str(line1).strip("@").split("_")
        barcode = str(attr[0])
        umi = str(attr[1])
        dic = {"readId": index, "barcode": barcode, "UMI": umi}
        read_row_list.append(dic)
        index += 1
    df_read = pd.DataFrame(read_row_list, columns=["readId", "barcode", "UMI"])
    mapping_vdj.logger.info("fq reads to dataframe done.")
    read2.close()
    total_read = df_read.shape[0]

    # init row list
    mapping_summary_row_list = []

    # mapped
    alignment = pd.read_csv(alignments, sep="\t")
    alignment.readId = alignment.readId.astype(int)
    align_read = alignment.shape[0]
    df_read.readId = df_read.readId.astype(int)
    df_align = pd.merge(df_read, alignment, on="readId", how="right")

    mapping_summary_row_list.append({
        "item": "Reads Mapped to Any VDJ Gene",
        "count": align_read,
        "total_count": total_read,
    })

    # CDR3
    df_CDR3 = df_align[~pd.isnull(df_align["aaSeqCDR3"])]
    align_read_with_CDR3 = df_CDR3.shape[0]
    mapping_summary_row_list.append({
        "item": "Reads with CDR3",
        "count": align_read_with_CDR3,
        "total_count": total_read,
    })

    # correct CDR3
    df_correct_CDR3 = df_CDR3[~(df_CDR3["aaSeqCDR3"].str.contains(r"\*"))]
    align_read_with_correct_CDR3 = df_correct_CDR3.shape[0]
    mapping_summary_row_list.append({
        "item": "Reads with Correct CDR3",
        "count": align_read_with_correct_CDR3,
        "total_count": total_read,
    })

    # VDJ
    df_VJ = df_correct_CDR3[
        (~pd.isnull(df_correct_CDR3['bestVGene'])) &
        (~pd.isnull(df_correct_CDR3['bestJGene']))
    ]
    df_VJ = df_VJ[df_VJ.bestVGene.str[:3] == df_VJ.bestJGene.str[:3]]
    df_VJ["chain"] = df_VJ.bestVGene.str[:3]
    df_VJ["VJ_pair"] = df_VJ["bestVGene"] + "_" + df_VJ["bestJGene"]
    Reads_Mapped_Confidently_to_VJ_Gene = df_VJ.shape[0]
    mapping_summary_row_list.append({
        "item": "Reads Mapped Confidently to VJ Gene",
        "count": Reads_Mapped_Confidently_to_VJ_Gene,
        "total_count": total_read
    })

    # chain
    for chain in chains:
        df_chain = df_VJ[df_VJ.chain == chain]
        Reads_Mapped_to_chain = df_chain.shape[0]
        mapping_summary_row_list.append({
            "item": f"Reads Mapped to {chain}",
            "count": Reads_Mapped_to_chain,
            "total_count": total_read,
        })

    # unique UMI
    df_UMI = df_VJ.drop_duplicates(subset=["barcode", "UMI"], keep="first")

    # filter1: keep top 1 in each combinations
    groupby_elements = [
        'barcode',
        'chain',
        'bestVGene',
        'bestJGene',
        'aaSeqCDR3',
        'nSeqCDR3',
    ]
    df_UMI_count = df_UMI.groupby(
        groupby_elements, as_index=False).agg({"UMI": "count"})
    df_UMI_count = df_UMI_count.sort_values("UMI", ascending=False)
    # out unfiltered
    df_UMI_count.to_csv(UMI_count_unfiltered_file, sep="\t", index=False)

    df_UMI_count_filter1 = df_UMI_count.groupby(
        ["barcode", "chain"], as_index=False).head(1)
    # out filtered1
    df_UMI_count_filter1.to_csv(
        UMI_count_filtered1_file,
        sep="\t",
        index=False)

    if debug:
        unique_UMI = df_UMI.shape[0]
        mapping_summary_row_list.append({
            "item": "UMI unique count",
            "count": unique_UMI,
            "total_count": align_read_with_correct_CDR3,
        })
        UMI_after_Contamination_Filtering = df_UMI_count.filter1.UMI.sum()
        mapping_summary_row_list.append({
            "item": "UMI after Contamination Filtering",
            "count": UMI_after_Contamination_Filtering,
            "total_count": unique_UMI,
        })

    # stat file
    df = pd.DataFrame(
        mapping_summary_row_list,
        columns=[
            "item",
            "count",
            "total_count"])
    stat_file = f'{outdir}/stat.txt'
    gen_stat(df, stat_file)

    # report
    STEP = 'mapping_vdj'
    name = f'{type}_{STEP}'
    t = reporter(
        name=name,
        sample=sample,
        stat_file=stat_file,
        outdir=outdir + '/..',
        assay=assay,
    )
    t.get_report()


@log
def mapping_vdj(args):
    sample = args.sample
    outdir = args.outdir
    fq = args.fq
    type = args.type
    debug = args.debug
    assay = args.assay
    thread = args.thread

    report = f"{outdir}/{sample}_align.txt"
    not_align_fq = f"{outdir}/not_align.fq"
    read2_vdjca = f"{outdir}/read2.vdjca"
    alignments = f"{outdir}/{sample}_alignments.txt"

    if not os.path.exists(outdir):
        os.system('mkdir -p %s' % outdir)

    cmd = f"""
mixcr align \
--species hs \
-t {thread} \
--not-aligned-R1 {not_align_fq} \
--report {report} \
-OallowPartialAlignments=true \
-OvParameters.geneFeatureToAlign=VTranscriptWithP \
{fq} \
{read2_vdjca}
mixcr exportAlignments \
{read2_vdjca} {alignments} \
-readIds --force-overwrite -vGene -dGene -jGene -cGene \
-nFeature CDR3 -aaFeature CDR3\n"""
    mapping_vdj.logger.info(cmd)
    os.system(cmd)

    # summary
    summary(fq, alignments, type, outdir, sample, assay, debug)


def get_opts_mapping_vdj(parser, sub_program):
    parser.add_argument("--type", help='TCR or BCR', required=True)
    parser.add_argument("--debug", action='store_true')
    if sub_program:
        parser.add_argument('--outdir', help='output dir', required=True)
        parser.add_argument('--sample', help='sample name', required=True)
        parser.add_argument("--fq", required=True)
        parser.add_argument("--thread", default=1)
        parser.add_argument('--assay', help='assay', required=True)



import unittest
import os
import pandas as pd
import glob
from celescope.tcr_fl.Barcode_index import Barcode_index
from celescope.tcr_fl.split_fq import *
from celescope.tcr_fl.assemble import *
from celescope.tools.utils import read_barcode_file, glob_genomeDir


class test_snp(unittest.TestCase):
    def setUp(self):
        os.chdir('/SGRNJ01/RD_dir/pipeline_test/zhouyiqi/unittest/tcr_fl/20201103')
        self.match_dir = '/SGRNJ01/RD_dir/pipeline_test/zhouyiqi/unittest/rna/test1'

    @unittest.skip('pass')
    def test_Barcode_index(self):
        barcodes, _nCell = read_barcode_file(self.match_dir)
        bi = Barcode_index(barcodes)
        bi.write_index('test_bi.tsv')
    
    @unittest.skip('pass')
    def test_get_nCell_barcodes(self):
        fq = 'TCR_sub_clean_2.fq.gz'
        barcodes = get_nCell_barcodes(fq, 50)
        print(barcodes)

    @unittest.skip('pass')
    def test_split_run(self):
        fq = '/SGRNJ02/RandD4/RD2019016/20201016/GOT0928_P1_T/02.cutadapt/GOT0928_P1_T_clean_2.fq.gz'
        fq_outdir = 'fastq'
        split_run(fq, fq_outdir, nCell=50)

    def test_assemble(self):
        os.chdir('/SGRNJ01/RD_dir/pipeline_test/zhouyiqi/unittest/tcr_fl/pipe/')
        fastq_dir = 'GOT0928_P1_T/03.split_fq/fastq'
        sample = 'GOT0928_P1_T'
        outdir = 'GOT0928_P1_T/04.assemble/'
        run_assemble(sample, outdir, fastq_dir, thread=4)


if __name__ == '__main__':
    unittest.main()


import unittest
import os
import pandas as pd
from celescope.tools.STAR import Step_mapping
from celescope.tools.utils import read_barcode_file, gene_convert
from .Chemistry import Chemistry

class Tests(unittest.TestCase):
    def setUp(self):
        os.chdir('/SGRNJ01/RD_dir/pipeline_test/zhouyiqi/unittest/rna')
        self.sample = 'test1'
        self.fq = './test1/02.cutadapt/test1_clean_2.fq.gz'
        self.genomeDir = '/SGRNJ/Public/Database/genome/homo_mus'
        self.thread = 4
        self.assay = 'rna'
        self.out_unmapped = False
        self.debug = True

    @unittest.skip('pass')
    def test_ribo(self):
        self.outdir = f'{self.sample}/03.STAR'
        mapping = Step_mapping(
            self.sample, 
            self.outdir, 
            self.assay, 
            self.thread,
            self.fq, 
            self.genomeDir, 
            self.out_unmapped, 
            self.debug)
        mapping.STAR_map_log = f'{self.outdir}/{self.sample}_Log.final.out'  
        mapping.STAR_map_log = f'{self.outdir}/{self.sample}_Log.final.out'
        mapping.picard_region_log = f'{self.outdir}/{self.sample}_region.log'
        mapping.ribo()
        mapping.format_stat()
        mapping.report()

    @unittest.skip('pass')
    def test_chemistry(self):
        fq1s = [
            '/SGRNJ/DATA_PROJ/2004/20201102_9/MG_201026_1-R2010283_L3_1.fq.gz',
            '/SGRNJ01/RD_dir/pipeline_test/zhouyiqi/unittest/test_data/smk/R2009217_L4_1.fq.gz',
            '/SGRNJ/DATA_PROJ/2003/20200727_6/R2007185_L3_1.fq.gz',
            '/SGRNJ/DATA_PROJ/2004/20201029/MG_201023_1-R2010226_L3_1.fq.gz',
        ]
        results = []
        for fq1 in fq1s:
            ch = Chemistry(fq1)
            results.append(ch.get_chemistry())
        print(results)
        assert results == ['scopeV2.1.1', 'scopeV2.1.1', 'scopeV2.0.1', 'scopeV2.2.1']      

    #@unittest.skip('pass')
    def test_gtf(self):
        gtf_file = '/SGRNJ/Database/script/genome/hs/gtf/Homo_sapiens.GRCh38.99.gtf'
        id_name = gene_convert(gtf_file)
        print(f"ENSG00000001629: {id_name['ENSG00000001629']}")

        gtf_file = '/SGRNJ01/RD_dir/pipeline_test/litao/genomes/Cricetulus_griseus/Cricetulus_griseus_crigri.CriGri_1.0.101.gtf'
        id_name = gene_convert(gtf_file)
        print(id_name)


if __name__ == '__main__':
    unittest.main()


from celescope.__init__ import __CONDA__
from celescope.capture_rna.__init__ import __STEPS__, __ASSAY__
from celescope.tools.Multi import Multi


class Multi_capture_rna(Multi):
    
    def custome_args(self):
        # parser
        self.STAR_args()
        self.analysis_args()
        self.parser.add_argument('--probe_file', help="probe fasta file")

    def read_custome_args(self):
        self.read_STAR_args()
        self.read_analysis_args()
        self.probe_file = self.args.probe_file

    def count_capture_rna(self, sample):
        step = 'count_capture_rna'
        bam = f'{self.outdir_dic[sample]["featureCounts"]}/{sample}_name_sorted.bam'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step } '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--bam {bam} '
            f'--match_dir {self.col4_dict[sample]} '
            f'--genomeDir {self.genomeDir} '
            f'--cells auto '
        )
        self.process_cmd(cmd, step, sample, m=10, x=1)
    
    def analysis(self, sample):
        step = 'analysis'
        matrix_file = f'{self.outdir_dic[sample]["count_capture_rna"]}/{sample}_matrix.tsv.gz'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step } '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--matrix_file {matrix_file} '
            f'{self.save_rds_str} '
            f'--type_marker_tsv {self.type_marker_tsv} '
        )
        self.process_cmd(cmd, step, sample, m=10, x=1)


def main():
    multi = Multi_capture_rna(__ASSAY__, __STEPS__, __CONDA__)
    multi.col4_default = None
    multi.run()

if __name__ == '__main__':
    main()







from celescope.__init__ import __CONDA__
from celescope.capture_virus.__init__ import __STEPS__, __ASSAY__
from celescope.tools.Multi import Multi


class Multi_capture_virus(Multi):
    def custome_args(self):
        self.STAR_args()
        self.parser.add_argument('--virus_genomeDir', help='virus_genomeDir', required=True)

    def read_custome_args(self):
        self.read_STAR_args()
        self.virus_genomeDir = self.args.virus_genomeDir

    def STAR_virus(self, sample):
        step = 'STAR_virus'
        input_read = f'{self.outdir_dic[sample]["cutadapt"]}/{sample}_clean_2.fq.gz'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step} '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--input_read {input_read} '
            f'--virus_genomeDir {self.virus_genomeDir} '
            f'--thread {self.thread} '
        )
        self.process_cmd(cmd, step, sample, m=self.starMem, x=self.thread)

    def count_capture_virus(self, sample):
        step = 'count_capture_virus'
        virus_bam = f'{self.outdir_dic[sample]["STAR_virus"]}/{sample}_virus_Aligned.sortedByCoord.out.bam'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step} '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--virus_bam {virus_bam} '
            f'--match_dir {self.col4_dict[sample]} '
        )
        self.process_cmd(cmd, step, sample, m=5, x=1)

    def analysis_capture_virus(self, sample):        
        step = 'analysis_capture_virus'
        virus_file = f'{self.outdir_dic[sample]["count_capture_virus"]}/{sample}_virus_UMI_count.tsv'
        cmd = (
            f'{self.__APP__} '
            f'{self.__ASSAY__} '
            f'{step} '
            f'--outdir {self.outdir_dic[sample][step]} '
            f'--sample {sample} '
            f'--assay {self.__ASSAY__} '
            f'--virus_file {virus_file} '
            f'--match_dir {self.col4_dict[sample]} '

        )
        self.process_cmd(cmd, step, sample, m=15, x=1)


def main():
    multi = Multi_capture_virus(__ASSAY__, __STEPS__, __CONDA__)
    multi.col4_default = None
    multi.run()


if __name__ == '__main__':
    main()



